// Extended Roots, by Domani Turner-Ward, ghostbirding.art@gmail.com, www.ghostbirding.art.
// April 30, 2023. 
// A wearable art piece - crocheted bracer adorned with LEDs controlled by nearby real-time soil moisture measurements.
// Transciever code based on https://iotdesignpro.com/projects/arduino-uno-with-nrf24l01-send-and-receive-temperature-and-humidity-data-wirelessly.
// Soil moisture sensor code based on https://www.makerguides.com/capacitive-soil-moisture-sensor-with-arduino/.
// Addressable LED code based on Simple example from Adafruit Neopixel Library (https://learn.adafruit.com/adafruit-neopixel-uberguide/arduino-library-installation) and the Brightness Test at https://projecthub.arduino.cc/Heathen_Hacks-v2/cb4f3fe4-0f25-492c-accb-c9effb05b7e7.

//Code for Arduino connected to the recieving transciever and programmable LEDs.

//Soil Moisture Capacitance Sensor:
const int OpenAirReading = 247; //MUST BE EDITED! Change to value displayed in transmitter's serial monitor when sensor is dry and suspended in air.
const int WaterReading = 220; //MUST BE EDITED! Change to value displayed in transmitter's serial monitor when sensor is suspended in water.
int MoisturePercent = 0;
int red;
int green;
int blue;

//Transciever:
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
#include <Wire.h> 
const uint64_t pipeIn = 0xE8E8F0F0E1LL; 
RF24 radio(7, 8); //  CN and CSN  pins of nrf
//Recognizes data.m, the data flowing from transmitter to receiver. Can create multiple, with different names; ex, byte z creates data.z
  struct MyData {
    byte m; 
  };
  MyData data;

//Addressable LEDs:
#include <Adafruit_NeoPixel.h>
#define LED_PIN   6 // The digital pin to which the LEDs' info wire is connected.
#define LED_COUNT 9 //Total number of LEDs in strip.
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

void setup(){
 //Reciever setup:
  Serial.begin(9600); 
  radio.begin();
  radio.setAutoAck(false);
  radio.setDataRate(RF24_250KBPS);
  radio.openReadingPipe(1, pipeIn);
  radio.startListening();
 //LED setup:
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)
}

void recvData(){ //Function for data reception.
  if ( radio.available() ) {
    radio.read(&data, sizeof(MyData));
    }
}

void loop(){
  recvData(); //Recieve data, as written in void recData
  MoisturePercent = map(data.m, OpenAirReading, WaterReading, 0, 100); //Uses calibration constants from beginning of code to create moisture percentage array, with OpenAirReading=0% and WaterReading=100%

  if (MoisturePercent >= 50){ //Define colors for when the percent moisture is greater than or equal to 50%
   red = 0; //No red is shown when percent moisture is greater than or equal to 50%
   blue = map(MoisturePercent, 50, 100, 0, 200); //Blue  starts minimal brightness at 50% and linearly increases to high brightness at maximal percent moisture
   green = map(MoisturePercent, 50, 75, 200, 0); // Green spans the median (50%) moisture levels.
   colorWipe(strip.Color(red,   green,   blue), 50); //Set color calculated by data and mapping.
}

if (MoisturePercent <= 50){ //Define colors for when the percent moisture is less than or equal to 50%
   red = map(MoisturePercent, 0, 50, 200, 0); //Red starts minimal brightness at 50% and linearly increases to high brightness at minimal percent moisture
   green = map(MoisturePercent, 50, 25, 200, 0); // Green spans the median (50%) moisture levels.
   blue = 0; //No blue is shown when percent moisture is less than or equal to 50%
   colorWipe(strip.Color(red,   green,   blue), 50); //Set color calculated by data and mapping.
 }

strip.show(); // Send the updated pixel colors to the LED strip.
  Serial.println(data.m); // Display current soil moisture capacitance measurement in Serial Monitor.
  delay(1000); //Pause for one second before returning to beginning of loop.
}

void colorWipe(uint32_t color, int wait) { //Function for colorWipe used in loop.
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}